require('prototypes');
var request = require('request');
var settings = require('./settings.js');
var fs = require('fs-extra');
var pathLibrary = require('path');
var upath = require('upath');

function getCredentials()
{
    return "?user=" + process.env.DANDELION_USERNAME + "&token=" + process.env.TOKEN;
}
function getResourceURL(path)
{
    return process.env.SERVER_ADDRESS + '/api/resources/' + upath.normalizeSafe(path) + getCredentials();
}
function getDirectoryURL(path)
{
    return process.env.SERVER_ADDRESS + '/api/resources/dirs/' + upath.normalizeSafe(path) + getCredentials();
}

function getStateURL()
{
    return process.env.SERVER_ADDRESS + '/api/state/' + getCredentials();
}

function getAbsolutePath(path)
{
    var absolutePath = upath.normalizeSafe((process.env.SERVER_FOLDER + path));
	return absolutePath;
}
function getLoginURL()
{
    return process.env.SERVER_ADDRESS + '/api/login/';
}
exports.copyResource = function(origin, destination, callback)
{
	fs.copy(origin, destination,callback);
};
exports.login = function(callback)
{
    var options =
    {
        url: getLoginURL(),
        body: {'user': process.env.DANDELION_USERNAME,
               'password': process.env.PASSWORD},
        json: true,
    };
    request.post(options, function(error, response, body)
    {
        if(error)
        {
            return callback("Error in login process: " + error);
        }
        if (response && response.statusCode != 200)
        {
            return callback("Error in login process: Status code" + response.statusCode);
        }
        process.env.TOKEN = body.token;
		process.env.DANDELION_USERNAME = body.username;
        console.log("Authenticated Successfully: ", process.env.TOKEN);
        return callback(null);
    });
};


exports.getServerFileList = function(callback)
{
    request.get(getStateURL(),function(error, response, body)
    {
        var serverList =[];
        if(body && response.statusCode == 200)
        {
            serverList = JSON.parse(body);
        }
        else
        {
            if (response)
            {
                console.log("Error, response from server: ", response.statusCode);
                error = response.statusCode;
            }
        }
        return callback(error, serverList);
    });
};
exports.uploadFile = function(path, callback)
{
	var options =
    {
        url: getResourceURL(path),
        headers: {
				'resource_kind': 'file',
				'owner': process.env.DANDELION_USERNAME,
				'mtime': Date.now(),
				'size':0,
				'parent':"",
			},
    };
    fs.createReadStream(getAbsolutePath(path))
        .on('error',function(err){
            console.log("Error reading file: %s %s", err);
        })
        .pipe(request.put(options))
		.on('response', function(response)
		{
			console.log('File', path, 'has been added');
    		if(response.statusCode != 201)
			{
				console.log("status code uploading response", response.statusCode);
			}
			return callback(null);
  		})
        .on('error', function(err){
            console.log("Error uploading file: %s %s",path, err);
        });
};

exports.getFile = function(path, callback)
{
    request(getResourceURL(path))
        .on('error', function(err){
            console.log("Error getting file: %s %s",path, err);
        })
        .pipe(fs.createWriteStream(getAbsolutePath(path)))
		.on('finish', function()
		{
			return callback(null);
  		})
        .on('error', function(err){
            console.log("Error writing file: %s %s",path, err);
        });
};


exports.deleteFile = function(path, callback)
{
    console.log("File", path, "has been deleted");
	var kind = "file";
	if (path.endsWith(pathLibrary.sep))
	{
		kind = "directory";
	}
	var options =
	{
		url: getResourceURL(path),
		headers: {
				'resource_kind': kind,
				'owner': process.env.DANDELION_USERNAME,
				'mtime': Date.now(),
				'size':'0',
			},
	};
    request.del(options,function(error, response, body)
    {
        if(error || response.statusCode != 204)
        {
            console.log("Error deleting from server:", path, error, response.statusCode);
            return callback(error);
        }
		return callback(null);
    });
};
exports.addDirectory = function(path, callback)
{
    console.log('DIRECTORY', path, 'has been added');
	var options =
    {
        url: getDirectoryURL(path),
        headers: {
				'resource_kind': 'file',
				'owner': process.env.DANDELION_USERNAME,
				'mtime': Date.now(),
				'size':'0',
			},
    };
    request.put(options,function(error, response, body)
    {
        if(error || response.statusCode != 200)
        {
            console.log("Error adding to server", path);
            return callback(error);
        }
		return callback(null);
    });
};

exports.deleteDirectory = function(path, callback)
{
    console.log('Dir', path, 'has been deleted');
    path += "/"; //Needed as WebDAV spec
    exports.deleteFile(path, callback);
};

exports.deleteLocalFile = function (path, callback)
{
    fs.remove(getAbsolutePath(path), function (error)
    {
        if (error)
        {
            console.log("error deleting file", error);
            return callback(error);
        }
        console.log('Successfully deleted', path);
		return callback(null);
    });
};

exports.deleteLocalDirectory = function (path, callback)
{
    fs.remove(getAbsolutePath(path), function (error)
    {
          if (error)
        {
            console.log("Error deleting directory:", error);
            return callback(error);
        }
          console.log('Successfully deleted: ',path);
		  return callback(null);
    });
};

exports.createLocalDirectory = function(path, callback)
{
    fs.mkdir(getAbsolutePath(path), function(error){
        if (error)
        {
            console.log("Error making directory:", error);
			return callback(error);
        }
        console.log('Successfully added:', path);
		return callback(null);
    });
};

exports.handleError = function (error)
{
    console.log("Error happened", error);
};

exports.logRawEvent = function (event, path, details)
{
//    console.log('Raw event info:', event, path, details);
};
